﻿Type=StaticCode
Version=1.60
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
'These global variables will be declared once when the application starts.
'These variables can be accessed from all modules.
'
'*************************************************************
'* Toolbar Code module                                       *
'*------------------------------------------------------------
'* Language:    Basic4Android                                *
'* Version:     1.0.0                                        *
'* Date:        2011/07/25                                   *
'* Author:      j.bergmann@gmx.com                           *
'* Description: an Easy-to-Use-Toolbar                       *
'*              needs only 5 Lines of Code in Sub Main       *
'*************************************************************
'   HOW TO USE:
'------------------------------------------------------------------------------------------------------------
'1. Add the Toolbar Code Module to your Project (Project->Add Existing Module)
'------------------------------------------------------------------------------------------------------------
'2. Add the following Images to your Files Folder:
'   toolbar_blank.png     (the Button Image without any Icon/Text)
'   toolbar_highlight.png (the highlighted Button Image without any Icon/Text, same size as toolbar_blank.png)
'   toolbar_texture.png   (the Toolbar Background Image, should have same Height as toolbar_blank.png)
'------------------------------------------------------------------------------------------------------------
'3. Add as many transparent Icons as you need, to your Files Folder (with the same size as toolbar_blank.png)
'   toolbar_1.png
'	toolbar_2.png
'	toolbar_3.png
'	etc...
'   
'   Optionally, add Icons for Two-State-Buttons, for Example:
'   toolbar_2a.png                (this will automatically make the second Button a 2-State Button)
'   etc...
'------------------------------------------------------------------------------------------------------------
'4. Add the following Code to your Sub Main:
'
'   Sub Globals
'       Dim cnv As Canvas                                        '1
'   End Sub
'
'   Sub Activity_Create(FirstTime As Boolean)
'       If FirstTime then Toolbar.Initialize                     '2
'       cnv.Initialize(Activity)                                 '3
'       Toolbar.Draw(Activity,cnv,0)                             '4
'   End Sub
'
'   Sub Activity_Touch(Action as Int, X As Float, Y As Float)
'       Toolbar.Touch(Activity,cnv,Action,x,y)                   '5
'   End Sub
'
'   Sub Toolbar_Click(ButtonIndex As Int)                        
'   End Sub                                                     
'
'   Sub Toolbar_Highlight(ButtonIndex As Int)                    
'   End Sub                                                      
'--------------------------------------------------------------------------------------------------------------

'***************************************
'Public Type Declarations
'***************************************
Type ToolbarButton(Name As String,bmp As Bitmap,bmpChecked As Bitmap, Checked As Boolean,Checkable As Boolean, Rectangle As Rect,m_Rectangle As Rect)
'Name:            Button Name, is set in AddButton Method
'bmp:             Holds the Button Icon, loaded from File 
'bmpChecked:      Holds the second Button Icon, loaded from File (for 2State Buttons)
'Checked:         Current Checked State (for 2-State Buttons)
'Checkable:       Boolean, Is this Button a 2-State-Button?
'Rectangle:       Button Rectangle on Activity
'm_Rectangle:     Button Rectangle on Memory Bitmap
''***************************************
'Private Variables
'****************************************
'The Prefix "m_" is used in this Module as a Workaround to mark private Code.
'Please do not access Variables and Subs with the Prefix "m_" from other Modules.
Dim m_bmp_tb0 As Bitmap             'Toolbar Texture                      (loaded from File) 
Dim m_bmp_bt1 As Bitmap    			'Toolbar Button Background            (loaded from File) 
Dim m_bmp_bt2 As Bitmap    			'Toolbar Button Background Highlight  (loaded from File)

Dim m_bmp_tb1 As Bitmap    			'Toolbar Normal                       (mutable Bitmap)
Dim m_bmp_tb2 As Bitmap    			'Toolbar Highlight                    (mutable Bitmap)
Dim m_bmp_tb3 As Bitmap             'Toolbar Normal(2State)               (mutable Bitmap) 
Dim m_bmp_tb4 As Bitmap             'Toolbar Highlight(2State)            (mutable Bitmap) 
	
Dim m_Blankfile As String     		'Image File Name (Default: toolbar_blank.png)
Dim m_HighlightFile As String     	'Image File Name (Default: toolbar_highlight.png)
Dim m_TextureFile As String     	'Image File Name (Default: toolbar_texture.png)
Dim m_Directory As String  			'Folder for Toolbar Images (Default: File.DirAssets)
Dim m_Scale As Float       			'Scale Factor for Resizing Buttons
Dim m_Alignment As Int                'Toolbar Gravity
Dim m_ButtonCount As Int 	     	'Number of Buttons

Dim m_ButtonHeight As Int  			'Scaled Button Height
Dim m_ButtonWidth As Int  		 	'Scaled Button Width 
Dim m_OriginalButtonHeight As Int 	'Original Button Bitmap Height(no Scaling)
Dim m_OriginalButtonWidth As Int  	'Original Button Bitmap Width (no Scaling)
	
Dim m_Rectangle As Rect             'Toolbar Rect  (Buttons only)
Dim m_FullRect As Rect              'Toolbar Rect  (incl. Texture) 
Dim m_Width As Int                  'Toolbar Width (Buttons only)
Dim m_Height As Int					'Toolbar Height
Dim m_Left As Int                   'Toolbar Left  (Buttons only)
Dim m_Top As Int					'Toolbar Top
Dim m_Right As Int                  'Toolbar Right (Buttons only)
Dim m_Bottom As Int                 'Toolbar Bottom

Dim m_ah As Int                     'Activity Height
Dim m_aw As Int                     'Activity Width
Dim m_IsInitialized As Boolean      'Initialized Flag
Dim m_LastButton As Int			    'Last highlighted Button
Dim m_MaxButtons As Int             'Maximum Number of Buttons
'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
'----------------------------------------------------------------------------------------------
m_MaxButtons = 5                    'You may change this Value, if you need more or less Buttons
'----------------------------------------------------------------------------------------------
'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Dim m_Buttons(m_MaxButtons-1) As ToolbarButton 

'*************************************************
'Public Constants
'*************************************************
Dim BUTTONSTATE_NORMAL As Int :  BUTTONSTATE_NORMAL=0
Dim BUTTONSTATE_HIGHLIGHT As Int:BUTTONSTATE_HIGHLIGHT=1
Dim BUTTONSTATE_CHECKED As Int:  BUTTONSTATE_CHECKED=2

Dim ACTION_DOWN As Int:          ACTION_DOWN=0
Dim ACTION_UP As Int:            ACTION_UP=1
Dim ACTION_MOVE As Int:          ACTION_MOVE=2
End Sub
'***************************************************
'Public Methods
'***************************************************
Sub Initialize()
'///There are two Initialization Methods (Initialize and Initialize1)
'///Easy-to-use Initialization,
'///Buttons are added automatically.
'///This Method should be called in Activity_Create (FirstTime = true)

Dim i As Int
Dim fname As String

	m_Directory=File.DirAssets
	m_BlankFile="toolbar_blank.png"
	m_HighLightFile="toolbar_highlight.png"
	m_TextureFile="toolbar_texture.png"
	m_Scale=1dip
	If m_ButtonCount>0 Then Clear
	
	'Load Images from File:
	m_loadToolbarImages
	
	'Set Initialized-Flag:
	m_IsInitialized =True
	
	'Add Buttons automatically
	AddButton(1,"","","")
	For i = 2 To m_MaxButtons
	    fname = "toolbar_" & NumberFormat(i,1,0) & ".png"
		If File.Exists(m_Directory,fname) = False Then Exit
		AddButton(i,"","","")
	Next
	
End Sub
Sub Initialize1(ImagesDirectory As String, BlankFile As String, HighlightFile As String, TextureFile As String, Scale As Float)
'///Use Initialization1 Method, when you want to use your own Image and Button File Names
'///Buttons are not added here, subsequent calls of Toolbar.AddButton are required
'///This Method should be called in the first Call of Activity_Create (FirstTime = true)
'///or at Runtime,for changing the Layout of the Toolbar.
	
	'Folder for Bitmaps, File.DirAssets, if ommitted:
	m_Directory=ImagesDirectory
	If m_Directory="" Then m_Directory=File.DirAssets
	
	'Filenames, use default names, if omitted
	m_BlankFile = Blankfile
	If m_BlankFile="" Then m_BlankFile="toolbar_blank.png"
	
	m_HighLightFile = HighLightfile
	If m_HighLightFile="" Then m_HighLightFile="toolbar_highlight.png"
	
	m_TextureFile = Texturefile
	If m_TextureFile="" Then m_TextureFile="toolbar_texture.png"
	
	'This is for scaling the Button Size on smaller devices, use a value between 0.1 and 2
	'Use 0 as a Default Value for Density Independent Pixel
	m_Scale = Scale
	
	If m_Scale=0 Then m_Scale=1dip
	If m_Scale<0.1 Then m_Scale=0.1
	If m_Scale>2 Then m_Scale =2
	
	If m_ButtonCount>0 Then Clear
		
	'Load Images from File:
	m_loadToolbarImages
	
	'Set Initialized-Flag:
	m_IsInitialized =True
End Sub
Sub Draw(Activity As Activity, cnv As Canvas, ToolbarGravity As Int)
'///(Re)draw the complete Toolbar
Dim cnv1 As Canvas
Dim r As Rect
Dim isRedraw As Boolean

	'This Method is called from Activity_Create 
	'Make sure Toolbar.Initialize Method AND 
	'Toolbar.AddButton Method are already called  
	'and cnv is initialized before the first
	'call of the Toolbar.Draw Method.
	
	m_ah = Activity.Height
	m_aw = Activity.Width
	
	Try
		If cnv.Bitmap.IsInitialized=False Then
			'Canvas Initialize missing
			cnv.Initialize(Activity)
		End If
	Catch
	    cnv.Initialize(Activity)
	End Try
	
	If m_IsInitialized = False Then
	   'Toolbar Initialize missing:
	   Initialize
	Else If m_Fullrect.isinitialized Then
	   'Redraw,erase previous Toolbar
	   isRedraw = True
	   cnv.DrawRect(m_FullRect,Colors.BLACK,True,1)
	   Activity.Invalidate2(m_FullRect)
	End If
	
	m_Alignment = ToolbarGravity
	If m_Alignment=0 Then m_Alignment = Bit.Or(Gravity.CENTER_HORIZONTAL,Gravity.BOTTOM)	
	
	'(Re)calculate Dimensions and Gravity
	m_CalcToolbarDimensions
	
		
	'(Re)draw Texture:
	m_DrawToolbarTexture(cnv)
	
	If m_ButtonCount >0 Then
		'Draw Toolbar:
		m_DrawToolbar (cnv1)
		'Draw Icons:
		m_DrawToolbarIcons(cnv1)
		'Draw the complete Toolbar on the provided Canvas:
		cnv.DrawBitmap(m_bmp_tb1,Null,m_Rectangle)
	End If
	
	If isRedraw Then
	   'keep Current State of 2-State-Buttons 
	   Refresh(Activity,cnv)
	Else   
	   'make Drawing visible:
	   Activity.Invalidate2(m_FullRect)
	End If
	
	m_LastButton=-1
End Sub
Sub AddButton(Index As Int, Name As String, Filename1 As String, Filename2 As String)
'///If you want to show different Layouts (Skins) at Runtime,
'///call this Method after Toolbar.Initialize1 for each Button
Dim i
    i=Index-1
	
	If Name="" Then Name = "Button " & NumberFormat(index,1,0)
	m_Buttons(i).Initialize
	m_Buttons(i).Name=Name

	If filename1="" Then FileName1="toolbar_" & NumberFormat(Index,1,0) & ".png"
	
	If File.Exists(m_Directory,Filename1)=False Then
	   m_CreateMissingBitmap(m_Buttons(i).bmp,Colors.Transparent)
	Else   
	   m_Buttons(i).bmp.Initialize(m_Directory,Filename1)
	End If   
	
	If filename2="" Then FileName2="toolbar_" & NumberFormat(Index,1,0) & "a.png"
	If File.Exists(m_Directory,filename2) Then
	   m_Buttons(i).bmpChecked.Initialize(m_Directory,filename2)
	   m_Buttons(i).Checkable=True
	Else 
	   m_Buttons(i).bmpChecked.Initialize3(m_Buttons(i).bmp)
	   m_Buttons(i).Checkable=False
	End If
	m_Buttons(i).Checked=False
	If index>m_ButtonCount Then m_ButtonCount = index
	
End Sub

Sub Touch(Activity As Activity, cnv As Canvas, Action As Int, x As Int,y As Int)As Boolean
'///this public Method should be called from Activity_Touch Event
'/// returns true, when x/y Coordinates are in Toolbar Rectangle
'///The Highlight Event will be raised, when a Button is touched
'///The Click Event will be raised, when a Button is clicked 
Dim hit As Int
Dim response As Int
    
	If m_IsInitialized = False Then Return
	If m_ButtonCount<=0 Then Return
	
	If m_PointinRect(x,y,m_Rectangle)=False Then
	   'not in ToolbarRectangle:
	   If m_LastButton>=0 Then
	      'Restore last Toolbar Button:
	      m_RestoreLastButton(Activity,cnv)
	    End If
	    Return False 'Inform the calling Module, x/y Coordinates are not in Toolbar Rect (Toolbar was not touched)
	End If
	
	'Test for Button Hit:
	hit=m_Hittest(x,y)
	   
	If hit>=0 Then 
		Select Case action
	       Case ACTION_DOWN, ACTION_MOVE
				If hit<>m_LastButton Then
				   m_RestoreLastButton(Activity,cnv)
				   m_DrawToolbarButton(cnv,hit,1)'Highlight
				   Activity.Invalidate2(m_Buttons(hit).Rectangle)
				   '------------------------------------
				   'Raise Highlight Event
				   CallSub2(Main,"Toolbar_HighLight",hit+1)
				   '-----------------------------------
				Else
				   'Button Highlight Event already handled
				End If
		   Case ACTION_UP
		        If m_Buttons(hit).Checkable Then
				   If m_Buttons(hit).Checked = True Then m_Buttons(hit).Checked= False Else m_Buttons(hit).Checked=True
				End If   
				m_DrawToolbarButton(cnv,hit,0)
				Activity.Invalidate2(m_Buttons(hit).Rectangle)
				m_LastButton=-1
				'----------------------------
				'Raise Click Event
				CallSub2(Main,"Toolbar_Click",hit+1)
				'-----------------------------
		    Case Else
				'
	    End Select
		Return True 'inform the Calling Module, x/y Coordinates are in Toolbar Rect (Toolbar was touched)
	 Else 'Hit =0 
	    'This should never happen
	 End If
	 
End Sub
Sub Clear()
'///Clear all Buttons
'///Use this Method, if you want to change existing Buttons at Runtime
'///Call Toolbar.clear before adding new Buttons with Toolbar.AddButtons Method
Dim i,u As Int
    u= m_MaxButtons-1
    For i= 0 To u
	    m_Buttons(i).initialize
	Next 
	m_ButtonCount=0
	m_LastButton=-1
End Sub
Sub Refresh(Activity As Activity, cnv As Canvas)
'///After changing Buttons programmatically,
'///call this Method to redraw the Toolbar
Dim i As Int
    For i = 0 To m_ButtonCount-1
	    m_DrawToolbarButton(cnv,i,0)
	Next 
	Activity.Invalidate2(m_Rectangle)
	m_LastButton=-1
End Sub
'*************************************************
'Public Read-Only Properties 
'*************************************************
Sub ButtonHeight() As Int
'///Returns the calculated ButtonHeight in Pixels(Scale Factor already applied)
	Return m_ButtonHeight
End Sub
Sub ButtonWidth() As Int
'///Returns the calculated ButtonHeight in Pixels(Scale Factor already applied)
    Return m_ButtonWidth
End Sub
Sub Width() As Int
'///returns the Width of the Toolbar (Buttons Only)
    Return m_Width
End Sub
Sub Height() As Int
'///returns the Height of the Toolbar (Buttons only)
    Return m_Height
End Sub
Sub Left() As Int
'///returns the left Position of the Toolbar(Buttons only)
    Return m_Left
End Sub
Sub Top() As Int
'///returns the Top Position of the Toolbar
    Return m_Top
End Sub
Sub Right() As Int
'///returns the right Position of the Toolbar (Buttons only)
    Return m_Right
End Sub
Sub Bottom() As Int
'///returns the Bottom Position of the Toolbar
    Return m_Bottom
End Sub
Sub Rectangle() As Rect
'///returns the Toolbar Rectangle (Buttons only
    Return m_Rectangle
End Sub
Sub IsInitialized() As Boolean
'///Initialization Flag, set in Sub Initialize
    Return m_IsInitialized
End Sub
Sub Alignment As Int
    Return m_Alignment
End Sub
Sub Buttons(index As Int)As ToolbarButton
    If index >m_ButtonCount Then index=m_ButtonCount
	If index<1 Then index=1
	Return m_Buttons(index-1)
End Sub

'****************************************************
' private Methods
'****************************************************
Sub m_LoadToolbarImages()
'///Loads the Toolbar Images from File
Dim fname As String
Dim i As Int

	'Load Button Background from File:
	fname = m_BlankFile '"toolbar_blank.png"
	If File.Exists(m_Directory,fname) Then
	   m_bmp_bt1.Initialize(m_Directory,fname)
	Else
	   m_CreateMissingBitmap(m_bmp_bt1,Colors.RED)
	End If
	
	'Calculate Toolbar Height:
	m_OriginalButtonHeight=m_bmp_bt1.Height
	m_OriginalButtonWidth=m_bmp_bt1.Width
	m_ButtonWidth= (m_OriginalButtonWidth*m_Scale)
	m_ButtonHeight= (m_OriginalButtonHeight*m_Scale)
	m_Height=m_ButtonHeight
	
	'Load Button Highlight from File:
	fname = m_HighLightFile '"toolbar_highlight.png"
	If File.Exists(m_directory,fname) Then
	   m_bmp_bt2.Initialize(m_Directory,fname)
    Else
	   m_CreateMissingBitmap(m_bmp_bt2,Colors.YELLOW)
	End If
	
    'Load Toolbar Texture
	fname = m_TextureFile'"toolbar_texture.png"
	If File.Exists(m_directory,fname) Then
	   m_bmp_tb0.Initialize(m_Directory,fname)
	Else
	   'Use blank button instead
	   m_bmp_tb0.Initialize3(m_bmp_bt1)
	End If 
End Sub
Sub m_CreateMissingBitmap(bmp As Bitmap, Color As Long)
'///to prevent Errors caused by missing Resource Files
Dim cnv As Canvas
Dim s As String
Dim r As Rect
Dim x,y,w,h As Int

    w=100
	h=100
    r.Initialize(0,0,w,h)
	bmp.InitializeMutable(w,h)
	cnv.Initialize2(bmp)
	cnv.DrawRect(r,Color,True,1)
	If color = Colors.Transparent Then 
	   'missing Icon
	   r.Initialize(0,0,w-1,h-1)
	   cnv.DrawRect(r,Colors.WHITE,False,1)
	   y=cnv.MeasureStringHeight("?",Typeface.DEFAULT,30)
	   x=cnv.MeasureStringWidth("?",Typeface.DEFAULT,30)
	   x=(w-x)/2
	   y=((h-y)/2)+y
	   cnv.DrawText("?",x,y,Typeface.DEFAULT,30,Colors.Black,"LEFT")
	End If   
End Sub
Sub m_CalcToolbarDimensions()
'///Calculate Toolbar Dimensions
'///and Gravity
    
	'Calculate Toolbar Width
	m_Width = m_ButtonCount*m_ButtonWidth
    'm_Height was already calculated in m_LoadToolbarImages
	
	'Calculate Gravity:
	'Calculate Vertikal Position
	Select Case True
	    Case Bit.And(m_Alignment,Gravity.Top) = Gravity.TOP
	        'Top:
			m_Top=0
			m_Bottom=m_Height
		Case Else
		    'Bottom (Default):
            m_Top = m_ah-m_Height                 
			m_Bottom = m_ah
	End Select
	
	'Calculate Horizontal Position:
	Select Case True
		Case Bit.And(m_Alignment,Gravity.Left) = Gravity.LEFT
		    'Left:
			m_Left=0
			m_Right=m_Width
		Case Bit.And(m_Alignment,Gravity.RIGHT) = Gravity.RIGHT
		    'Right:
			m_Left = (m_aw-m_width)
			m_Right=m_aw
		Case Else
			'Center (Default):
			m_Left = (m_aw-m_Width)/2             
	        m_Right = m_Left+m_width
	End Select
	
	m_Rectangle.Initialize(m_Left,m_Top,m_Right,m_Bottom)
	
End Sub
Sub m_DrawToolBarButton(cnv As Canvas,index As Int,state As Int)
'///Draw a single Button: 
	Select Case state
	       Case BUTTONSTATE_NORMAL'0
	            If m_Buttons(index).checked Then
				   cnv.drawBitmap(m_bmp_tb3,m_Buttons(index).m_Rectangle,m_Buttons(index).Rectangle)
				Else
				   cnv.drawBitmap(m_bmp_tb1,m_Buttons(index).m_Rectangle,m_Buttons(index).Rectangle)
				End If
	       Case BUTTONSTATE_HIGHLIGHT'1
		        If m_Buttons(index).checked Then
				   cnv.drawBitmap(m_bmp_tb4,m_Buttons(index).m_Rectangle,m_Buttons(index).Rectangle)
				Else
				   cnv.drawBitmap(m_bmp_tb2,m_Buttons(index).m_Rectangle,m_Buttons(index).Rectangle)
				End If	
				m_LastButton=index
		   Case Else
	End Select
End Sub
Sub m_DrawToolBar(cnv1 As Canvas)
'///Draws Toolbar Background on four mutable Bitmaps,
'///Icon Drawing happens in m_DrawToolbarIcons
Dim r As Rect 
Dim i,n,bw As Int    
	
	If m_ButtonCount=0 Then Return	
	bw=m_ButtonWidth     
	bh=m_ButtonHeight   
	
	m_bmp_tb1.InitializeMutable(m_Width,bh)   'Toolbar Normal Bitmap 
	m_bmp_tb2.InitializeMutable(m_Width,bh)   'Toolbar Highlight Bitmap
	m_bmp_tb3.InitializeMutable(m_Width,bh)   'Toolbar Normal (2State) Bitmap	
	m_bmp_tb4.InitializeMutable(m_Width,bh)   'Toolbar Highlight (2State) Bitmap
	
	
	n=m_ButtonCount-1
	
	'Draw Toolbar Normal Bitmap 
    cnv1.Initialize2(m_bmp_tb1)
	For  i = 0 To n
	     r.Initialize(i*bw,0,(i*bw)+bw,bh)
	     cnv1.drawbitmap(m_bmp_bt1,Null,r)
	Next
	
	'Draw Toolbar Highlight Bitmap
	cnv1.Initialize2(m_bmp_tb2)
	For  i = 0 To n
	     r.Initialize(i*bw,0,(i*bw)+bw,bh)
	     cnv1.drawbitmap(m_bmp_bt2,Null,r)
	Next
	'Draw Toolbar Normal 2State Bitmap
	m_bmp_tb3.Initialize3(m_bmp_tb1)
	
	'Draw Toolbar Higlight 2State Bitmap
	m_bmp_tb4.Initialize3(m_bmp_tb2)
	
End Sub
Sub m_DrawToolbarIcons(cnv1 As Canvas)
'///Draws Button Icons on four Toolbar Bitmaps:    
Dim i,n As Int 
Dim bw, bh, bt As Int
	
	If m_ButtonCount=0 Then Return
	bw=m_ButtonWidth
	bh=m_ButtonHeight
	bt=m_Rectangle.Top
	
	'1. Draw Buttons on normal Toolbar
	cnv1.Initialize2(m_bmp_tb1)
	For i = 1 To m_ButtonCount
	    n=i-1
		m_Buttons(n).m_Rectangle.Initialize((n*bw),0,(n*bw)+bw,bh)
		m_Buttons(n).Rectangle.Initialize(m_Left +(n*bw),bt,m_Left+(n*bw)+bw,bt+bh)
		cnv1.DrawBitmap(m_Buttons(n).bmp,Null,m_Buttons(n).m_Rectangle)
	Next 
	'2. Draw Buttons on highlighted Toolbar
	cnv1.Initialize2(m_bmp_tb2)
	For i = 1 To m_ButtonCount
	    n=i-1
		cnv1.DrawBitmap(m_Buttons(n).bmp,Null,m_Buttons(n).m_Rectangle)
	Next
	'3. Draw 2-State Buttons on normal Toolbar (2State)
	cnv1.Initialize2(m_bmp_tb3)
	For i = 1 To m_ButtonCount
	    n=i-1
		cnv1.DrawBitmap(m_Buttons(n).bmpChecked,Null,m_Buttons(n).m_Rectangle)
	Next
	'4. Draw 2-State-Buttons on highlighted Toolbar (2State)
	cnv1.Initialize2(m_bmp_tb4)
	For i = 1 To m_ButtonCount
	    n=i-1
		cnv1.DrawBitmap(m_Buttons(n).bmpChecked,Null,m_Buttons(n).m_Rectangle)
	Next
End Sub
Sub m_DrawToolbarTexture(cnv As Canvas)
'///Draw a Texture on the the Toolbar Region
'///The blank Button Image will be used as Default
'///if Texture Filename is missing.
   m_FullRect.Initialize(0,m_top,m_aw,m_top+m_height)
   cnv.DrawBitmap(m_bmp_tb0,Null,m_FullRect)
    
End Sub
Sub m_RestoreLastButton(Activity As Activity,cnv As Canvas)
'///Restore the last highlighted Button
    If m_LastButton<0 Then Return
	
	m_DrawToolbarButton(cnv,m_LastButton,0)
	Activity.Invalidate2(m_Buttons(m_LastButton).Rectangle)
	m_LastButton=-1
End Sub
Sub m_HitTest(x As Int,y As Int)As Int
'///Helper Function for Toolbar.Touch Method
Dim i,n As Int
    For i = 1 To m_ButtonCount
	    n=i-1
		If m_PointInRect(x,y,m_Buttons(n).Rectangle) Then
		   Return n
		End If
	Next
	Return -1
End Sub
Sub m_PointInRect(x As Int,y As Int,r As Rect) As Boolean
'///Helper Function for Toolbar.Touch Method
	If (x>=r.Left AND x<=r.Right AND y>=r.Top AND y<=r.Bottom) Then 
		Return True
	Else
		Return False
	End If
End Sub


